# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Qt implementation of the UI."""

from functools import wraps

# pylint: disable=F0401,E0611
from PyQt4.QtCore import Qt, pyqtSignal
from PyQt4.QtGui import (
    QApplication,
    QCursor,
    QFrame,
    QFontMetrics,
    QHBoxLayout,
    QVBoxLayout,
    QStyle,
    QWizardPage,
    QLabel,
)
from twisted.internet import defer

from ubuntu_sso import main
from ubuntu_sso.logger import setup_gui_logging
from ubuntu_sso.qt import PREFERED_UI_SIZE, TITLE_STYLE
from ubuntu_sso.utils.ui import GENERIC_BACKEND_ERROR


logger = setup_gui_logging('ubuntu_sso.sso_wizard_page')


class Header(QFrame):
    """Header Class for Title and Subtitle in all wizard pages."""

    def __init__(self):
        """Create a new instance."""
        super(Header, self).__init__()
        self.setObjectName('header')
        vbox = QVBoxLayout(self)
        vbox.setContentsMargins(0, 0, 0, 0)
        self.title_label = QLabel()
        self.title_label.setWordWrap(True)
        self.title_label.setObjectName('title_label')
        self.subtitle_label = QLabel()
        self.subtitle_label.setWordWrap(True)
        self.subtitle_label.setFixedHeight(32)
        vbox.addWidget(self.title_label)
        vbox.addWidget(self.subtitle_label)
        self.title_label.setVisible(False)
        self.subtitle_label.setVisible(False)

    def set_title(self, title):
        """Set the Title of the page or hide it otherwise"""
        if title:
            fm = QFontMetrics(self.subtitle_label.font())
            width = PREFERED_UI_SIZE['width'] * 0.95
            elided_text = fm.elidedText(title, Qt.ElideRight, width)
            self.title_label.setToolTip(title)
            self.title_label.setText(elided_text)
            self.title_label.setVisible(True)
        else:
            self.title_label.setVisible(False)

    def set_subtitle(self, subtitle):
        """Set the Subtitle of the page or hide it otherwise"""
        if subtitle:
            fm = QFontMetrics(self.subtitle_label.font())
            width = PREFERED_UI_SIZE['width'] * 1.8
            elided_text = fm.elidedText(subtitle, Qt.ElideRight, width)
            self.subtitle_label.setText(elided_text)
            self.subtitle_label.setToolTip(subtitle)
            self.subtitle_label.setVisible(True)
        else:
            self.subtitle_label.setVisible(False)


class SSOWizardPage(QWizardPage):
    """Root class for all wizard pages."""

    ui_class = None
    _signals = {}  # override in children
    processingStarted = pyqtSignal()
    processingFinished = pyqtSignal()

    def __init__(self, app_name, **kwargs):
        """Create a new instance."""
        parent = kwargs.pop('parent', None)
        super(SSOWizardPage, self).__init__(parent=parent)

        self.ui = None
        if self.ui_class is not None:
            # self.ui_class is not callable, pylint: disable=E1102
            self.ui = self.ui_class()
            self.ui.setupUi(self)

        # store common useful data provided by the app
        self.app_name = app_name
        self.ping_url = kwargs.get('ping_url', '')
        self.tc_url = kwargs.get('tc_url', '')
        self.policy_url = kwargs.get('policy_url', '')
        self.help_text = kwargs.get('help_text', '')

        # Set the error area
        self.form_errors_label = QLabel(' ')
        self.form_errors_label.setObjectName('form_errors')
        self.form_errors_label.setAlignment(Qt.AlignBottom)
        self.layout().insertWidget(0, self.form_errors_label)
        # Set the header
        self.header = Header()
        self.header.set_title(title='')
        self.header.set_subtitle(subtitle='')
        self.layout().insertWidget(0, self.header)
        self._signals_receivers = {}
        self.backend = None

        self.layout().setAlignment(Qt.AlignLeft)

        self.setup_page()

    def show_error(self, app_name, message):
        """Show an error message inside the page."""
        self.hide_overlay()
        fm = QFontMetrics(self.form_errors_label.font())
        width = PREFERED_UI_SIZE['width'] * 0.95
        elided_text = fm.elidedText(message, Qt.ElideRight, width)
        self.form_errors_label.setText(elided_text)
        self.form_errors_label.setToolTip(message)

    def hide_error(self):
        """Hide the label errors in the current page."""
        # We actually want the label with one chat, because if it is an
        # empty string, the height of the label is 0
        self.form_errors_label.setText(' ')

    def hide_overlay(self):
        """Emit the signal to notify the upper container that ends loading."""
        self.setEnabled(True)
        self.processingFinished.emit()

    def show_overlay(self):
        """Emit the signal to notify the upper container that is loading."""
        self.setEnabled(False)
        self.processingStarted.emit()

    @defer.inlineCallbacks
    def setup_page(self):
        """Setup the widget components."""
        client = yield main.get_sso_client()
        self.backend = client.sso_login

        self._setup_signals()
        self._set_translated_strings()
        self._connect_ui()

    # pylint: disable=C0103

    def cleanupPage(self):
        """Hide the errors."""
        self.hide_error()

    def setTitle(self, title=''):
        """Set the Wizard Page Title."""
        self.header.set_title(TITLE_STYLE % title)

    def setSubTitle(self, subtitle=''):
        """Set the Wizard Page Subtitle."""
        self.header.set_subtitle(subtitle)

    def title(self):
        """Return the header's title."""
        return self.header.title_label.text()

    def subTitle(self):
        """Return the header's subtitle."""
        return self.header.subtitle_label.text()

    # pylint: enable=C0103

    def _filter_by_app_name(self, f):
        """Excecute the decorated function only for 'self.app_name'."""

        @wraps(f)
        def inner(app_name, *args, **kwargs):
            """Execute 'f' only if 'app_name' matches 'self.app_name'."""
            result = None
            if app_name == self.app_name:
                result = f(app_name, *args, **kwargs)
            else:
                logger.info('%s: ignoring call since received app_name '\
                            '"%s" (expected "%s")',
                            f.__name__, app_name, self.app_name)
            return result

        return inner

    def _setup_signals(self):
        """Bind signals to callbacks to be able to test the pages."""
        for signal, method in self._signals.iteritems():
            actual = self._signals_receivers.get(signal)
            if actual is not None:
                msg = 'Signal %r is already connected with %r.'
                logger.warning(msg, signal, actual)

            match = self.backend.connect_to_signal(signal, method)
            self._signals_receivers[signal] = match

    def _set_translated_strings(self):
        """Implement in each child."""
        raise NotImplementedError()

    def _connect_ui(self):
        """Implement in each child."""
        raise NotImplementedError()

    def _handle_error(self, remote_call, handler, error):
        """Handle any error when calling the remote backend."""
        logger.error('Remote call %r failed with: %r', remote_call, error)
        errordict = {'message': GENERIC_BACKEND_ERROR}
        handler(self.app_name, errordict)


class EnhancedLineEdit(object):
    """Represents and enhanced lineedit.

    This class works on an already added lineedit to the widget so
    that we are just adding extra items to it.
    """

    def __init__(self, line_edit, valid_cb=lambda x: False,
                 warning_sign=False):
        """Create an instance."""
        super(EnhancedLineEdit, self).__init__()
        self._line_edit = line_edit
        layout = QHBoxLayout(self._line_edit)
        layout.setMargin(0)
        self._line_edit.setLayout(layout)
        self.valid_cb = valid_cb
        layout.addStretch()
        self.clear_label = QLabel(self._line_edit)
        self.clear_label.setMargin(2)
        self.clear_label.setProperty("lineEditWarning", True)
        layout.addWidget(self.clear_label)
        self.clear_label.setMinimumSize(16, 16)
        self.clear_label.setVisible(False)
        self.clear_label.setCursor(QCursor(Qt.ArrowCursor))
        if warning_sign:
            icon = QApplication.style().standardIcon(
                QStyle.SP_MessageBoxWarning)
            self.clear_label.setPixmap(icon.pixmap(16, 16))
        # connect the change of text to the cation that will check if the
        # text is valid and if the icon should be shown.
        self._line_edit.textChanged.connect(self.show_button)

    def show_button(self, string):
        """Decide if we show the button or not."""
        if not self.valid_cb(string) and self.clear_label.pixmap() is not None:
            self.clear_label.setVisible(True)
        else:
            self.clear_label.setVisible(False)


class SSOWizardEnhancedEditPage(SSOWizardPage):
    """Page that contains enhanced line edits."""

    # Method '_connect_ui', '_set_translated_strings' is abstract in class
    # 'SSOWizardPage' but is not overridden
    # pylint: disable=W0223

    def __init__(self, *args, **kwargs):
        """Create a new instance."""
        self._enhanced_edits = {}
        super(SSOWizardEnhancedEditPage, self).__init__(*args, **kwargs)

    def set_line_edit_validation_rule(self, edit, cb):
        """Set a new enhanced edit so that we can show an icon."""
        if edit in self._enhanced_edits:
            self._enhanced_edits[edit].valid_cb = cb
        else:
            # create a new enhanced edit
            enhanced_edit = EnhancedLineEdit(edit, cb)
            self._enhanced_edits[edit] = enhanced_edit
