# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Test the SSOWizardPage and related."""

from twisted.internet import defer

from ubuntu_sso.qt import PREFERED_UI_SIZE
from ubuntu_sso.qt.setup_account_page import SetupAccountPage
from ubuntu_sso.qt.sso_wizard_page import Header
from ubuntu_sso.qt.tests import BaseTestCase, PageBaseTestCase


class HeaderTest(BaseTestCase):

    """Tests for injected Header in each Wizard Page."""

    @defer.inlineCallbacks
    def setUp(self):
        yield super(HeaderTest, self).setUp()
        self.header = Header()

    def test_label_state(self):
        """Check the title and subtitle properties."""
        self.assertTrue(self.header.title_label.wordWrap())
        self.assertTrue(self.header.subtitle_label.wordWrap())
        self.assertFalse(self.header.title_label.isVisible())
        self.assertFalse(self.header.subtitle_label.isVisible())

    def test_set_title(self):
        """Check if set_title works ok, showing the widget if necessary."""
        self.header.set_title('title')
        self.assertEqual(self.header.title_label.text(), 'title')
        self.header.show()
        self.assertTrue(self.header.title_label.isVisible())
        self.header.hide()

    def test_set_elided_title(self):
        """Check if set_title adds the ellipsis when necessary."""
        # add an extra letter so we ensure this needs to be trimmed
        title = 'a' * int(PREFERED_UI_SIZE['width'] * 0.95) + 'a'
        self.header.set_title(title)
        self.assertEqual(self.header.title_label.toolTip(), title)
        expected = unicode(self.header.title_label.text())
        self.assertTrue(expected.endswith(u'\u2026'))

    def test_set_empty_title(self):
        """Check if the widget is hidden for empty title."""
        self.header.set_title('')
        self.assertFalse(self.header.title_label.isVisible())

    def test_set_subtitle(self):
        """Check if set_subtitle works ok, showing the widget if necessary."""
        self.header.set_subtitle('subtitle')
        self.assertEqual(self.header.subtitle_label.text(), 'subtitle')
        self.header.show()
        self.assertTrue(self.header.subtitle_label.isVisible())
        self.header.hide()

    def test_set_elided_subtitle(self):
        """Check if set_subtitle adds the ellipsis when necessary."""
        subtitle = 'a' * int(PREFERED_UI_SIZE['width'] * 0.95) + 'a'
        self.header.set_subtitle(subtitle)
        self.assertEqual(self.header.subtitle_label.toolTip(), subtitle)
        expected = unicode(self.header.subtitle_label.text())
        self.assertTrue(expected.endswith(u'\u2026'))

    def test_set_empty_subtitle(self):
        """Check if the widget is hidden for empty subtitle."""
        self.header.set_title('')
        self.assertFalse(self.header.title_label.isVisible())


class SSOWizardPageTest(PageBaseTestCase):

    """Tests for SSOWizardPage."""

    ui_class = SetupAccountPage

    def test_show_error(self):
        """Test show_error with a normal length string."""
        message = 'error-message'
        self.ui.show_error(self.app_name, message)
        self.assertEqual(self.ui.form_errors_label.toolTip(), message)
        expected = unicode(self.ui.form_errors_label.text())
        self.assertEqual(expected, message)

    def test_show_error_long_text(self):
        """Test show_error with a long length string."""
        message = 'a' * int(PREFERED_UI_SIZE['width'] * 0.95) + 'a'
        self.ui.show_error(self.app_name, message)
        self.assertEqual(self.ui.form_errors_label.toolTip(), message)
        expected = unicode(self.ui.form_errors_label.text())
        self.assertTrue(expected.endswith(u'\u2026'))

    def test_hide_error(self):
        """Test show_error with a long length string."""
        message = ' '
        self.ui.hide_error()
        self.assertEqual(self.ui.form_errors_label.text(), message)
